/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_UTIL_EIGEN_TYPES_H_INCLUDED
#define CNOID_UTIL_EIGEN_TYPES_H_INCLUDED

#include <Eigen/Core>
#include <Eigen/Geometry>
//#include <Eigen/AlignedVector3>

namespace cnoid {

    using Eigen::Matrix2d;
    using Eigen::Vector2d;
    using Eigen::Matrix3d;
    using Eigen::Vector3d;
    using Eigen::Matrix4d;
    using Eigen::Vector4d;
    using Eigen::MatrixXd;
    using Eigen::VectorXd;
    using Eigen::AngleAxisd;
    using Eigen::Quaterniond;

    typedef Eigen::Matrix2d Matrix2;
    typedef Eigen::Vector2d Vector2;
    typedef Eigen::Matrix3d Matrix3;
    typedef Eigen::Vector3d Vector3;
    //typedef Eigen::AlignedVector3<double> Vector3;
    typedef Eigen::Matrix4d Matrix4;
    typedef Eigen::Vector4d Vector4;
    typedef Eigen::Matrix<double, 6, 1> Vector6;
    typedef Eigen::Affine3d Affine3;
    typedef Eigen::AngleAxisd AngleAxis;
    typedef Eigen::Quaterniond Quat;


    class SE3 {
        Quat q;
        Vector3 p;
    public:
        inline SE3() { }
        inline SE3(const Vector3& translation, const Quat& rotation)
            : p(translation), q(rotation) { }
        inline void set(const Vector3 translation, const Quat& rotation) {
            this->p = translation;
            this->q = rotation;
        }
        inline void set(const Vector3 translation, const Matrix3& R) {
            this->p = translation;
            this->q = R;
        }
        inline Vector3& translation() { return p; }
        inline const Vector3& translation() const { return p; }
        inline Quat& rotation() { return q; }
        inline const Quat& rotation() const { return q; }

        EIGEN_MAKE_ALIGNED_OPERATOR_NEW        
    };

}

#endif
